@@ -11,6 +11,8 @@ from eth_account.datastructures import SignedMessage
 from eth_account.messages import encode_defunct
 from eth_account.signers.local import LocalAccount
 from pydantic import NoneStr, PositiveInt
+from requests.adapters import HTTPAdapter
+from urllib3.util import Retry
 
 from farcaster.config import *
 from farcaster.models import *
@@ -45,6 +47,14 @@ class MerkleApiClient:
         self.expires_at = expires_at
         self.rotation_duration = rotation_duration
         self.session = requests.Session()
+        self.session.mount(
+            self.config.base_path,
+            HTTPAdapter(
+                max_retries=Retry(
+                    total=2, backoff_factor=1, status_forcelist=[520, 413, 429, 503]
+                )
+            ),
+        )
         if self.access_token:
             self.session.headers.update(
                 {"Authorization": f"Bearer {self.access_token}"}
@@ -445,6 +455,33 @@ class MerkleApiClient:
         )
         return FollowingGetResponse(**response).result
 
+    def get_all_following(self, fid: Optional[int] = None) -> UsersResult:
+        """Get all the users a user is following by iterating through the next cursors
+
+        Args:
+            fid (int): Farcaster ID of the user
+
+        Returns:
+            UsersResult: model containing users
+        """
+        users: List[ApiUser] = []
+        cursor = None
+        limit = 100
+        if fid is None:
+            fid = self.get_me().fid
+        while True:
+            response = self._get(
+                "following",
+                params={"fid": fid, "cursor": cursor, "limit": limit},
+            )
+            response_model = FollowingGetResponse(**response)
+            if response_model.result.users:
+                users.extend(response_model.result.users)
+            if response_model.next is None:
+                break
+            cursor = response_model.next.cursor
+        return UsersResult(users=users)
+
     def follow_user(self, fid: PositiveInt) -> StatusContent:
         """Follow a user
 
@@ -477,18 +514,18 @@ class MerkleApiClient:
         )
         return StatusResponse(**response).result
 
-    def get_me(self) -> UserResult:
+    def get_me(self) -> ApiUser:
         """Get the current user
 
         Returns:
-            UserResult: model containing the current user
+            ApiUser: model containing the current user
         """
         response = self._get(
             "me",
         )
         response_model = MeGetResponse(**response).result
         self.config.username = response_model.user.username
-        return response_model
+        return response_model.user
 
     def get_mention_and_reply_notifications(
         self,
@@ -585,56 +622,56 @@ class MerkleApiClient:
         )
         return StatusResponse(**response).result
 
-    def get_user(self, fid: int) -> UserResult:
+    def get_user(self, fid: int) -> ApiUser:
         """Get a user
 
         Args:
             fid (int): Farcaster ID of the user
 
         Returns:
-            UserResult: model containing the user
+            ApiUser: model containing the user
         """
         response = self._get(
             "user",
             params={"fid": fid},
         )
-        return UserGetResponse(**response).result
+        return UserGetResponse(**response).result.user
 
     def get_user_by_username(
         self,
         username: str,
-    ) -> UserResult:
+    ) -> ApiUser:
         """Get a user by username
 
         Args:
             username (str): username of the user
 
         Returns:
-            UserResult: model containing the user
+            ApiUser: model containing the user
         """
         response = self._get(
             "user-by-username",
             params={"username": username},
         )
-        return UserByUsernameGetResponse(**response).result
+        return UserByUsernameGetResponse(**response).result.user
 
     def get_user_by_verification(
         self,
         address: str,
-    ) -> UserResult:
+    ) -> ApiUser:
         """Get a user by verification address
 
         Args:
             address (str): address of the user
 
         Returns:
-            UserResult: model containing the user
+            ApiUser: model containing the user
         """
         response = self._get(
             "user-by-verification",
             params={"address": address},
         )
-        return UserByUsernameGetResponse(**response).result
+        return UserByUsernameGetResponse(**response).result.user
 
     def get_user_collections(
         self,
